@@ -1,6 +1,6 @@
 package Auto::Mata;
 # ABSTRACT: A simple, reliable finite state machine
-$Auto::Mata::VERSION = '0.07';
+$Auto::Mata::VERSION = '0.08';
 
 use strict;
 use warnings;
@@ -51,7 +51,11 @@ sub machine (&) {
   #-----------------------------------------------------------------------------
   # Define the machine parameters
   #-----------------------------------------------------------------------------
-  my %fsm = (ready => undef, term => undef, map => {});
+  my %fsm = (
+    ready => undef,
+    term  => undef,
+    map   => {},
+  );
 
   do {
     local $_ = \%fsm;
@@ -64,6 +68,11 @@ sub machine (&) {
   my $term     = $fsm{term};
   my $Terminal = declare 'Terminal', as Tuple[Enum[$term], Any];
 
+  #-----------------------------------------------------------------------------
+  # For each state, compile a type union that matches all 'on' constraints
+  # where that state is the 'to' state. This will be used to validate the state
+  # accumulator after each transition to that state.
+  #-----------------------------------------------------------------------------
   my %final = ($term => $Terminal);
 
   foreach my $from (keys %map) {
@@ -89,7 +98,7 @@ sub machine (&) {
 
       push @match, $initial, sub {
         my ($from, $input) = @$_;
-        debug('%s -> %s: %s', $from, $to, explain($input));
+        debug('%s -> %s', $from, $to);
 
         do { local $_ = $input; $input = $with->() } if $with;
         my $state = [$to, $input];
@@ -101,7 +110,8 @@ sub machine (&) {
 
           croak join "\n",
             sprintf('Transition from %s to %s produced an invalid state.', $from, $to),
-            sprintf('Attempted to move from %s to %s', explain($_), explain($state)),
+            sprintf('Initial state: %s', explain($_)),
+            sprintf('Final state: %s', explain($_)),
             sprintf($error);
         }
 
@@ -167,7 +177,7 @@ sub transition ($%) {
   debug("New state: $init");
 
   foreach my $next (@{$_->{map}{$from}}) {
-    croak "identical transition $next->{initial} already defined"
+    croak "transition conflict in $from to $to: $on already matched by $next->{initial}"
       if $init == $next->{initial};
   }
 
@@ -275,7 +285,7 @@ Auto::Mata - A simple, reliable finite state machine
 
 =head1 VERSION
 
-version 0.07
+version 0.08
 
 =head1 SYNOPSIS
 
@@ -462,8 +472,6 @@ L<Exporter/Specialised-Import-Lists>).
 
   use Auto::Mata '!with';
 
-=back
-
   machine {
     ...
 
@@ -479,6 +487,8 @@ L<Exporter/Specialised-Import-Lists>).
       };
   };
 
+=back
+
 =head1 DEBUGGING
 
 If C<$ENV{DEBUG_AUTOMATA}> is true, helpful debugging messages will be emitted
